%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                 MSB2_product.m
%                   Russell Luke
%               Department of MathSci
%              University of Delaware  
%                  April 12, 2008 
%
%
%
%
% DESCRIPTION:  TO DO
% 
%  Stable MSB2 matrix product
%
% INPUT:
%     K,mem: dimension variables
%     u_vec, the vector to be multiplied
%     S_mat, the matrix of steps
%     Y_mat, the matrix of gradient differences
%     del,   the initial scaling
%     alpha, a regularization parameter
%
% OUTPUT:
%        
%     v_vec = H*u_vec, where H is the MSB2 matrix
%
% USAGE: 
%
%       v_vec = MSB2_product(K,mem,u_vec, S_mat, Y_mat, del, alpha)
% 
% SUBROUTINE CALLS:
%      none
%      

function v_vec = MSB2_product(K,mem,u_vec, S_mat, Y_mat, del, alpha)

% gbar=[norm(Y_mat(1:K/2,mem));norm(Y_mat(K/2+1:K,mem))];
% Omega=[gbar(2)/gbar(1)*speye(K/2,K/2), sparse(K/2,K/2);sparse(K/2,K/2),speye(K/2,K/2)];
% Y_mat=Omega*Y_mat;
% S_mat=Omega*S_mat;
% u_vec=Omega*u_vec;
ny=zeros(mem,1);
for j=1:mem
    ny(j)=1/norm(Y_mat(:,j));
end
Y_mat=Y_mat*diag(ny);
S_mat=S_mat*diag(ny);

tmpzw = (Y_mat'*Y_mat+alpha*eye(mem,mem))\(Y_mat'*u_vec);
Wtmp_vec = u_vec;
Ztmp_vec = S_mat*tmpzw;
for j=0:mem-2
    tmpw=tmpzw;
    Wtmp_vec = Wtmp_vec-Y_mat(:,1+j:mem)*tmpw;
    tmpzw=(Y_mat(:,2+j:mem)'*Y_mat(:,2+j:mem)+alpha*eye(mem-(j+1),mem-(j+1)))\(Y_mat(:,2+j:mem)'*Wtmp_vec);
    Ztmp_vec = Ztmp_vec + S_mat(:,2+j:mem)*tmpzw;
end
Wtmp_vec = Wtmp_vec - Y_mat(:,mem)*tmpzw;
v_vec = del*Wtmp_vec + Ztmp_vec;
% v_vec=Omega\v_vec;
