%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                    RL_gradRL.m
%                   Russell Luke
%             Department of MathSci
%              University of Delaware  
%                    Oct 3, 2007 
% 
%    INPUT: 
%           params, a structure file
%           x, a matrix
%
%    OUTPUT:
%           RL, a scalar function value
%          gradRL, the gradient of RL, a vector
%
%      USAGE: 
%            [RL, gradRL]=RL_gradRL(params,x)
%
function [RL, gradRL]=RL_gradRL(params,x)

dim=params.dim;
im=sqrt(-1);
epsilon=1e-24;
c=1e+8;
scalesq=1/(dim(1)*dim(2));
P=complex(zeros(dim(1),dim(2),dim(3)));
K=complex(zeros(dim(1),dim(2),dim(3)));
R=complex(zeros(dim(1),dim(2),1+dim(3)));
J=zeros(1+dim(3),1);
gradRL = zeros(dim(1),dim(2),2);
temp=feval('reshape',x,dim(1),2*dim(2));
x=zeros(dim(1),dim(2),2);
x(:,:,1)=temp(:,1:dim(2));
x(:,:,2)=temp(:,dim(2)+1:2*dim(2));

% ======== Pupil Plane =========
% R(:,:,1) = (x(:,:,1).*x(:,:,1)+x(:,:,2).*x(:,:,2)).^(.5)-Xi_A;
temp= x(:,:,1).*x(:,:,1)+x(:,:,2).*x(:,:,2);
R(:,:,1)=temp./((temp+epsilon).^(.5))-params.Xi_A;
% R(:,:,1) = (x(:,:,1).*x(:,:,1)+x(:,:,2).*x(:,:,2))-Xi_A;
J(1)=sum(sum(R(:,:,1).*R(:,:,1)));

% ======== Image plane =========
for j=1:dim(3)

    P(:,:,j)=feval('FFT',...
        (x(:,:,1)+im*x(:,:,2)).*params.Xi_A.*exp(im*params.aberration(:,:,j)));
    % actually P^
    K(:,:,j)=real(P(:,:,j).*conj(P(:,:,j))); % technically
    % K(-^).^(.5)
    % I'm working with k^(.5) here.
    R(:,:,j+1)=K(:,:,j)./((K(:,:,j)+epsilon).^(.5))-params.k(:,:,j);
    J(1+j) = feval('sum',feval('sum',R(:,:,j+1).*R(:,:,j+1)));
end
J=.5*J;
RL=sum(log(J+c));%*scalesq;
% ============end function evaluations==============

% ==========================
% =  compute the gradient  =
% ==========================

% ========== pupil plane ===========
% temp=((R(:,:,1)+Xi_A)./(temp+epsilon)).*(1+2*epsilon./temp);
temp= (temp+2*epsilon)./((temp+epsilon).^(1.5));
tempu=(R(:,:,1).*(x(:,:,1).*temp));
tempu=tempu+1i*(R(:,:,1).*(x(:,:,2).*temp));
gradRL(:,:,1) = real(tempu)./(J(1)+c); 
gradRL(:,:,2) = imag(tempu)./(J(1)+c); 

% ========== Image plane ===========
tempu=complex(zeros(dim(1),dim(2)));
tmpgradRL=complex(zeros(dim(1),dim(2)));
for j=1:dim(3)
    temp=(K(:,:,j)+2*epsilon)./((K(:,:,j)+epsilon).^(1.5));
    tempu=(params.Xi_A.*exp(-im*params.aberration(:,:,j)).*...
        feval('IFFT',R(:,:,j+1).*(P(:,:,j).*temp)));
    tmpgradRL = tmpgradRL + tempu./(J(1+j)+c); 
end
gradRL(:,:,1)=(gradRL(:,:,1)+real(tmpgradRL))*scalesq;
gradRL(:,:,2)=(gradRL(:,:,2)+imag(tmpgradRL))*scalesq;
clear tempu 

% =========end gradient calculations=========
temp(1:dim(1),1:dim(2))= gradRL(:,:,1);
temp(1:dim(1),dim(2)+1:2*dim(2))= gradRL(:,:,2);
gradRL=feval('reshape',temp,dim(1)*2*dim(2),1);


clear R P K temp
clear params x

