from numpy import complex_, exp, imag, log, real, reshape, sum, zeros

from problems.JWST.fft import FFT, IFFT


def RL_gradRL(params, x):
    dim = params['dim']
    im = 1j
    epsilon = 1e-24
    c = 1e+8
    scalesq = 1./(dim[0]*dim[1])
    P = zeros(dim, dtype=complex_)
    K = zeros(dim, dtype=complex_)
    R = zeros((dim[0], dim[1], dim[2]+1), dtype=complex_)
    J = zeros(1+dim[2], dtype=complex_)
    gradRL = zeros((dim[0], dim[1], 2))
    temp = reshape(x, (dim[0], -1), order='F')
    x = zeros((dim[0], dim[1], 2))
    x[:, :, 0] = temp[:, :dim[1]]
    x[:, :, 1] = temp[:, dim[1]:2*dim[1]]

    # ======= Pupil Plane =======
    temp = x[:, :, 0]**2 + x[:, :, 1]**2
    R[:, :, 0] = temp/((temp+epsilon)**(.5)) - params['Xi_A']
    J[0] = sum(sum(R[:, :, 0]**2))

    # ======= Image plane =======
    # PYTHON PORT ANNOTATION
    # Changed loop variable j to k, as j is the imaginary unit in Python.
    for k in range(dim[2]):
        P[:, :, k] = FFT((x[:, :, 0] + im*x[:, :, 1]) *
                         params['Xi_A']*exp(im*params['aberration'][:, :, k]))
        K[:, :, k] = real(P[:, :, k]*P[:, :, k].conj())
        R[:, :, k+1] = K[:, :, k]/((K[:, :, k] + epsilon)**(.5))\
                       - params['k'][:, :, k]
        J[k+1] = sum(sum(R[:, :, k+1]**2))

    J = .5*J
    RL = real(sum(log(J+c)))
    # ========== end function evaluations ==========

    # ========================
    # = compute the gradient =
    # ========================

    # ======= pupil plane =======
    temp = (temp + 2*epsilon)/((temp + epsilon)**1.5)
    tempu = R[:, :, 0]*(x[:, :, 0]*temp)
    tempu = tempu + im*(R[:, :, 0]*(x[:, :, 1]*temp))
    gradRL[:, :, 0] = real(tempu/(J[0]+c))
    gradRL[:, :, 1] = imag(tempu/(J[0]+c))

    # ======= Image plane =======
    tempu = zeros((dim[0], dim[1]), dtype=complex_)
    tmpgradRL = zeros((dim[0], dim[1]), dtype=complex_)
    for k in range(dim[2]):
        temp = (K[:, :, k] + 2*epsilon)/((K[:, :, k] + epsilon)**1.5)
        tempu = params['Xi_A']*exp(-im*params['aberration'][:, :, k])\
                * IFFT(R[:, :, k+1]*(P[:, :, k]*temp))
        tmpgradRL = tmpgradRL + tempu/(J[k+1] + c)

    gradRL[:, :, 0] = scalesq*(gradRL[:, :, 0] + real(tmpgradRL))
    gradRL[:, :, 1] = scalesq*(gradRL[:, :, 1] + imag(tmpgradRL))
    # ========== end gradient calculations ==========
    temp = zeros((dim[0], 2*dim[1]))
    temp[:dim[0], :dim[1]] = gradRL[:, :, 0]
    temp[:dim[0], dim[1]:2*dim[1]] = gradRL[:, :, 1]
    gradRL = reshape(temp, 2*dim[0]*dim[1], order='F')

    return RL, gradRL
