import sys
sys.path.append('../../')

from numpy import array, ones
from numpy.linalg import norm

from samsara import Samsara, BFGS1_product, Explicit_TR, cdSY_mat
from problems.Rosenbrock.rosenbrock import Rosenbrock1000


def driver():
    Objective = Rosenbrock1000
    x0_vec = .09*ones(1000)
    xold_vec = x0_vec
    fold, gradfold_vec = Objective(xold_vec)

    options = {}
    options['verbose'] = True
    options['alpha_0'] = 5e-9
    options['gamma'] = .5
    options['c'] = .01
    options['beta'] = .9999999
    options['eta1'] = .995
    options['eta2'] = .8
    options['eta3'] = .25
    options['maxmem'] = 8
    options['tr'] = 1e+15

    options['QN_method'] = BFGS1_product
    options['Step_finder'] = Explicit_TR
    options['History'] = cdSY_mat
    options['update_conditions'] = 'Trust Region'
    options['initial_step'] = 1e-7

    s = Samsara(**options)

    # OPTIONS
    ngrad_TOL = 1e-11
    step_TOL = 1e-14
    Maxit = 500

    it = 0

    xnew_vec = array([])
    stepsize = 999.
    fnew = None
    gradfnew_vec = array([])
    ngradfnew = 999.

    while it < Maxit and ngradfnew > ngrad_TOL and stepsize > step_TOL:
        xnew_vec, xold_vec, fold, gradfold_vec, stepsize =\
            s.run(xold_vec, xnew_vec, fold, fnew,
                  gradfold_vec, gradfnew_vec)

        it += 1
        fnew, gradfnew_vec = Objective(xnew_vec)
        ngradfnew = norm(gradfnew_vec)

    if stepsize <= step_TOL:
        print('Algorithm stagnated:  stepsize tolerance violated.')

    xopt_vec = xnew_vec
    fopt = fnew

    s.plot(fopt=fopt)

    print('fopt =', fopt)

    return xopt_vec, fopt


if __name__ == '__main__':
    driver()
