"""Tests for the functions from step_finder.py.

Note
----
The tests are currently not working, as you need parameters,
that have very specific dependencies.
"""
import sys
import os

# Add samsara to the path.
PACKAGE_PARENT = '..'
SCRIPT_DIR = os.path.dirname(os.path.realpath(os.path.join(
    os.getcwd(), os.path.expanduser(__file__))))
sys.path.append(os.path.normpath(os.path.join(SCRIPT_DIR, PACKAGE_PARENT)))

from numpy import array, abs, random, zeros
from numpy.linalg import norm
import matlab
import matlab.engine
from samsara import *
from samsara.scale import scale_wrapper, delta_invBFGS
from samsara.step_finder import step_wrapper
from samsara.history import history_wrapper, cdSY_mat, fdSY_mat
from problems.Burke.burke import Burke_func

# The upper boundary for the allowed error between the MATLAB and
# Python implementations.
EPS = 1e-8

Hessian_functions = [BFGS1_product, Broyden1_product, Broyden2_product]
invHessian_functions = [invBFGS1_product, invBroyden1_product,
                        invBroyden2_product]

eng = matlab.engine.start_matlab()
eng.addpath('../../matlab')
eng.addpath('../../matlab/test_problems/Burke')


def gen_params():
    """
    Generate random parameters using the Burke function
    """
    # TODO(benedikt) adjust values to not take sqrt of negative value in Dogleg_QN
    dim = 7
    maxmem = random.randint(5, 12)
    mem = random.randint(2, maxmem-1)
    x_mat = zeros((dim, mem+1))
    gradf_mat = zeros((dim, mem+1))
    for i in range(mem+1):
        x = random.rand(dim)
        x_mat[:, i] = x
        f, grad_vec = Burke_func(x)
        gradf_mat[:, i] = grad_vec
    grad_vec = gradf_mat[:, -2]
    ngrad = norm(grad_vec)
    S_mat, Y_mat = history_wrapper(cdSY_mat, dim, mem+1, x_mat,
                                   gradf_mat)
    print('S_mat.shape =', S_mat.shape)
    print('Y_mat.shape =', Y_mat.shape)
    tr = 2.*random.ranf() * ngrad
    alpha = 5e-8
    rm = scale_wrapper(delta_invBFGS, dim, mem, S_mat, Y_mat, alpha)

    return dim, mem, grad_vec, ngrad, S_mat, Y_mat, tr, rm, alpha


def test_Dogleg_QN():
    for Hessian_product in Hessian_functions:
        for invHessian_product in invHessian_functions:
            for i in range(10):
                print('iteration:', i)
                dim, mem, grad_vec, ngrad, S_mat, Y_mat, tr, rm,\
                    alpha = gen_params()
                grad_vec_m = eng.transpose(matlab.double(grad_vec.tolist()))
                S_mat_m = matlab.double(S_mat.tolist())
                Y_mat_m = matlab.double(Y_mat.tolist())
                res_mat = eng.Dogleg_QN(Hessian_product.__name__,
                                        invHessian_product.__name__,
                                        dim, mem, grad_vec_m, ngrad,
                                        S_mat_m, Y_mat_m, tr, rm,
                                        alpha, 0)
                # res_py = Dogleg_QN(Hessian_product,
                #                    invHessian_product, dim, dim,
                #                    array(grad_vec).flatten(), ngrad,
                #                    array(S_mat), array(Y_mat), tr,
                #                    rm, alpha)
                res_py = step_wrapper(Dogleg_QN, Hessian_product,
                                      invHessian_product, dim, mem,
                                      grad_vec, ngrad, S_mat, Y_mat,
                                      tr, rm, alpha, True)
                print('res_mat')
                print(res_mat)
                print('res_py')
                print(res_py)
                diff = array(res_mat).flatten() - res_py.flatten()
                assert max(abs(diff)) < EPS


# def test_Explicit_TR():
#     for Hessian_product in Hessian_functions:
#         for invHessian_product in invHessian_functions:
#             for _ in range(10):
#                 dim, mem, grad_vec, ngrad, S_mat, Y_mat, tr, rm,\
#                     alpha = gen_params()
#                 grad_vec_m = eng.transpose(matlab.double(grad_vec.tolist()))
#                 S_mat_m = matlab.double(S_mat.tolist())
#                 Y_mat_m = matlab.double(Y_mat.tolist())
#                 res_mat = eng.Explicit_TR(Hessian_product.__name__,
#                                           invHessian_product.__name__,
#                                           dim, mem, grad_vec_m, ngrad,
#                                           S_mat_m, Y_mat_m, tr, rm,
#                                           alpha, 0)
#                 # res_py = Explicit_TR(invHessian_product, dim, mem,
#                 #                      array(grad_vec), ngrad,
#                 #                      array(S_mat), array(Y_mat), tr,
#                 #                      rm, alpha, True)
#                 res_py = step_wrapper(Explicit_TR, Hessian_product,
#                                       invHessian_product, dim, mem,
#                                       grad_vec, ngrad, S_mat, Y_mat,
#                                       tr, rm, alpha, True)
#                 diff = array(res_mat).flatten() - res_py.flatten()
#                 assert max(abs(diff)) < EPS
